#ifndef __PREPROC_HEADER_INCLUDED__
#define __PREPROC_HEADER_INCLUDED__

/* We call this with the config args, and the plugin
 * adds a preprocessor function on to the active list */
typedef int (*proc_preproc_init)(char *);

/* A preprocessor */
struct preproc {
	struct preproc		*next;
	const char		*name;
	proc_preproc_init	init;
};

/* Actually does packet preprocessing */
#define PREPROC_OK	0
#define PREPROC_DROP	1
typedef int (*proc_preprocess)(struct packet *, unsigned int, void *);
typedef void (*proc_preproc_free)(void *);

/* An active preprocessor instance */
struct pp_active {
	proc_preprocess process;
	proc_preproc_free free;
	void *priv;
};


#ifndef __PLUGIN__
void preproc_init(void);
void preproc_cleanup(int, void *);
struct preproc *preproc_find(const char *);
int preproc_load(char *, char *);
#endif

#ifndef __PLUGIN__
void preproc_dispatch(struct packet *);
int preproc_add(const char *, proc_preproc_init);
int preproc_activate(char *, proc_preprocess, proc_preproc_free, void *);
#else
typedef void(*proc_preproc_dispatch)(struct packet *);
typedef int (*proc_preproc_add)(const char *, proc_preproc_init);
typedef int(*proc_preproc_activate)(char *, proc_preprocess, proc_preproc_free, void *);
#endif

#endif /* __PREPROC_HEADER_INCLUDED__ */
