#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <errno.h>

#include <packet.h>
#include <signature.h>
#include <matcher.h>
#include <template.h>
#include <plugin.h>

proc_template template_shortrange;

int iseq_match(struct packet *p, void *priv, unsigned int l, int n)
{
	struct pkt_icmphdr *icmph=p->layer[l+1].h.icmp;
	u_int16_t seq=__constant_ntohs(icmph->un.echo.seq);
	struct shortrange *lr=priv;

	if(icmph->type != ICMP_ECHO && icmph->type != ICMP_ECHOREPLY)
		return n^0;
	
	return n ^ (seq>=lr->min && seq<=lr->max);
}

int iid_match(struct packet *p, void *priv, unsigned int l, int n)
{
	struct pkt_icmphdr *icmph=p->layer[l+1].h.icmp;
	u_int16_t id=__constant_ntohs(icmph->un.echo.id);
	struct shortrange *lr=priv;

	if(icmph->type != ICMP_ECHO && icmph->type != ICMP_ECHOREPLY)
		return n^0;
	
	return n ^ (id>=lr->min && id<=lr->max);
}

proc_match_match iseq_validate(char *arg, void **ptr, struct criteria *m)
{
	if ( template_shortrange(arg, ptr) ) {
		return iseq_match;
	}else return NULL;
}

proc_match_match iid_validate(char *arg, void **ptr, struct criteria *m)
{
	if ( template_shortrange(arg, ptr) ) {
		return iid_match;
	}else return NULL;
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	int ok=0;
	proc_matcher_add matcher_add;
	
	plugin_check(in, out);
	
	PLUGIN_ID("match.icmp", "ICMP matching routines");
	PLUGIN_VERSION(2, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");

	if ( !(matcher_add=in->import("matcher.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}
	
	if ( !(template_shortrange=in->import("template.shortrange")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=matcher_add("icmp_id",
		iid_validate,
		NULL,
		in->import("template.ptr"));
	
	ok+=matcher_add("icmp_seq",
		iseq_validate,
		NULL,
		in->import("template.ptr"));

	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
