#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <errno.h>

#include <packet.h>
#include <signature.h>
#include <matcher.h>
#include <template.h>
#include <plugin.h>

proc_template template_longrange;

#define MODE_NORMAL	0
#define MODE_ALL	1
#define MODE_NOT	2
#define MODE_ANY	3
struct tcp_flags{
	u_int8_t	flags;
	u_int8_t	mode;
};

int seq_match(struct packet *p, void *priv, unsigned int l, int n)
{
	u_int32_t seq=__constant_ntohl(p->layer[l+1].h.tcp->seq);
	struct longrange *lr=priv;
	return n ^ (seq>=lr->min && seq<=lr->max);
}

int ack_match(struct packet *p, void *priv, unsigned int l, int n)
{
	u_int32_t seq=__constant_ntohl(p->layer[l+1].h.tcp->ack);
	struct longrange *lr=priv;
	return n ^ (seq>=lr->min && seq<=lr->max);
}

proc_match_match ack_validate(char *arg, void **ptr, struct criteria *m)
{
	if ( template_longrange(arg, ptr) )
		return ack_match;
	else
		return NULL;
}

proc_match_match seq_validate(char *arg, void **ptr, struct criteria *m)
{
	if ( template_longrange(arg, ptr) )
		return seq_match;
	else
		return NULL;
}

int flags_match(struct packet *p, void *priv, unsigned int l, int n)
{
	struct tcp_flags *x=priv;
	u_int8_t flags=p->layer[++l].h.tcp->flags.flags;

	if ( x->mode==MODE_NORMAL) return n ^ (flags==x->flags);
	if ( x->mode==MODE_ANY) return n ^ ((flags & x->flags) != 0);
	if ( x->mode==MODE_ALL) return n ^ ((flags & x->flags) == x->flags);
	if ( x->mode==MODE_NOT) return n ^ ((flags & x->flags) == 0);
	
	printf("match_tcp: Wrong TCP mode. Should never happen!\n");
	return 0;
}

proc_match_match flags_validate(char *args, void **priv, struct criteria *m)
{
	struct tcp_flags *p;
	char *tmp;
	
	if ( !(p=calloc(1, sizeof(*p))) ) return NULL;

	*priv=p;
	
	for(tmp=args; *tmp; tmp++) {
		switch(*tmp) {
			case 'f':
			case 'F':
				p->flags |=  TCP_FIN;
				break;

			case 's':
			case 'S':
				p->flags |= TCP_SYN;
				break;
				
			case 'r':
			case 'R':
				p->flags |= TCP_RST;
				break;

			case 'p':
			case 'P':
				p->flags |= TCP_PSH;
				break;

			case 'a':
			case 'A':
				p->flags |= TCP_ACK;
				break;

			case 'u':
			case 'U':
				p->flags |= TCP_URG;
				break;

			case 'E':
			case 'e':
			case '2':
				p->flags |= TCP_ECE;
				break;

			case 'C':
			case 'c':
			case '1':
				p->flags |= TCP_CWR;
				break;

			case '0':
				p->flags = 0;
				break;

			case '!':
				p->mode = MODE_NOT;
				break;

			case '*':
				p->mode = MODE_ANY;
				break;

			case '+':
				p->mode = MODE_ALL;
				break;

			case ' ':
			case '\t':
			case ',':
				break;

			default:
				free(p);
				return NULL;
		}
	}
	
	return flags_match;
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	int ok=0;
	proc_matcher_add matcher_add;
	
	plugin_check(in, out);
	
	PLUGIN_ID("match.tcp", "TCP matching routines");
	PLUGIN_VERSION(2, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");

	if ( !(matcher_add=in->import("matcher.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}
	
	if ( !(template_longrange=in->import("template.longrange")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=matcher_add("flags",
		flags_validate,
		NULL,
		in->import("template.ptr"));

	ok+=matcher_add("seq",
		seq_validate,
		NULL,
		in->import("template.ptr"));
	
	ok+=matcher_add("ack",
		ack_validate,
		NULL,
		in->import("template.ptr"));

	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
